package org.exoplatform.services.portletcontainer.impl;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Locale;
import java.util.Map;
import java.util.ResourceBundle;
import java.util.Set;

import javax.portlet.PortletMode;
import javax.portlet.WindowState;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.xml.namespace.QName;

import org.exoplatform.container.ExoContainer;
import org.exoplatform.container.ExoContainerContext;
import org.exoplatform.services.portletcontainer.PortletContainerConf;
import org.exoplatform.services.portletcontainer.PortletContainerException;
import org.exoplatform.services.portletcontainer.PortletContainerPlugin;
import org.exoplatform.services.portletcontainer.PortletContainerService;
import org.exoplatform.services.portletcontainer.pci.ActionInput;
import org.exoplatform.services.portletcontainer.pci.ActionOutput;
import org.exoplatform.services.portletcontainer.pci.EventInput;
import org.exoplatform.services.portletcontainer.pci.EventOutput;
import org.exoplatform.services.portletcontainer.pci.Input;
import org.exoplatform.services.portletcontainer.pci.RenderInput;
import org.exoplatform.services.portletcontainer.pci.RenderOutput;
import org.exoplatform.services.portletcontainer.pci.ResourceInput;
import org.exoplatform.services.portletcontainer.pci.ResourceOutput;
import org.picocontainer.Startable;

/**
 * Created by the Exo Development team. Author : Mestrallet Benjamin
 * benjamin.mestrallet@exoplatform.com
 */
public class PortletContainerServiceImpl implements PortletContainerService,  Startable {


  private ArrayList<PortletContainerPlugin>             plugins;
  private ExoContainer          container;
  private PortletContainerConf  config;
  
  public PortletContainerServiceImpl(ExoContainerContext context) {
    this.plugins = new ArrayList<PortletContainerPlugin>();
    this.container = context.getContainer();
    this.config = (PortletContainerConf) container.getComponentInstanceOfType(PortletContainerConf.class);
  }

  public void addPlugin(PortletContainerPlugin plugin) {
    System.out.println(" PortletContainerServiceImpl plugin registered: " + plugin);
    plugins.add(plugin);
  }
  
  public void start() {
  }

  public void stop() {
  }

  public void setName(String name) {
    config.setName(name);
  }

  public void setDescription(String description) {
    config.setDescription(description);
  }
  
  public void setMajorVersion(int version){
    config.setMajorVersion(version);
  }

  public void setMinorVersion(int version){
    config.setMinorVersion(version);
  }

  public void setProperties(Map properties) {
    config.setProperties(properties);
  }

  public Collection getSupportedPortletModes() {
    ArrayList<PortletMode> res = new ArrayList<PortletMode>();
    for (Object plugin : plugins.toArray()) {
      res.addAll(((PortletContainerPlugin)plugin).getSupportedPortletModes());
    }
    return res;
  }

  public Collection getSupportedWindowStates() {
    ArrayList<WindowState> res = new ArrayList<WindowState>();
    for (Object plugin : plugins.toArray()) {
      res.addAll(((PortletContainerPlugin)plugin).getSupportedWindowStates());
    }
    return res;
  }

  public Collection getPortletModes(String portletAppName, String portletName,
      String markup) {
    try {
      return findPluginByPAPPName(portletAppName).
        getPortletModes(portletAppName, portletName, markup);
    } catch (Exception e) {
      e.printStackTrace();
    }
    return null;
  }

  public boolean isModeSuported(String portletAppName, String portletName,
      String markup, PortletMode mode) {
    return findPluginByPAPPName(portletAppName).
        isModeSuported(portletAppName, portletName, markup, mode);
  }

  public Collection getWindowStates( String portletAppName,
                                     String portletName,
                                     String markup) {
    try {
      return findPluginByPAPPName(portletAppName).
        getWindowStates(portletAppName, portletName, markup);
    } catch (Exception e) {
      e.printStackTrace();
    }
    return null;

  }

  public boolean isStateSupported(String portletAppName, String portletName,
      String markup, WindowState state) {
    return findPluginByPAPPName(portletAppName).
        isStateSupported(portletAppName, portletName, markup, state);
  }

  public Map getAllPortletMetaData() {
    HashMap res = new HashMap();
    for (Object plugin : plugins.toArray()) {
      res.putAll(((PortletContainerPlugin)plugin).getAllPortletMetaData());
    }
    return res;
  }

  public ResourceBundle getBundle(HttpServletRequest request,
      HttpServletResponse response, String portletAppName, String portletName,
      Locale locale) throws PortletContainerException {
    return findPluginByPAPPName(portletAppName).
        getBundle(request, response, portletAppName, portletName, locale);
  }

  protected PortletContainerPlugin findPluginByPAPPName(String papp) {
    for (Object plugin : plugins.toArray()) {
      Map plts = ((PortletContainerPlugin)plugin).getAllPortletMetaData();
      Set keys = plts.keySet();
      Iterator i = keys.iterator();
      while(i.hasNext()) {
        String s = (String) i.next();
        if (s.startsWith(papp + "/"))
          return (PortletContainerPlugin)plugin;
      }
    }
    return null;
  }
  
  public void setPortletPreference(Input input, Map preferences)
      throws PortletContainerException {
    PortletContainerPlugin plugin = findPluginByPAPPName(input.getInternalWindowID().getPortletApplicationName());
    plugin.setPortletPreference(input, preferences);
  }

  public Map getPortletPreference(Input input) {
    PortletContainerPlugin plugin = findPluginByPAPPName(input.getInternalWindowID().getPortletApplicationName());
    return plugin.getPortletPreference(input);
  }

  public ActionOutput processAction(HttpServletRequest request,
      HttpServletResponse response, ActionInput input)
      throws PortletContainerException {
    return findPluginByPAPPName(input.getInternalWindowID().
        getPortletApplicationName()).processAction(request, response, input);
  }


  public EventOutput processEvent(HttpServletRequest request,
      HttpServletResponse response, EventInput input)
      throws PortletContainerException {
    return findPluginByPAPPName(input.getInternalWindowID().
        getPortletApplicationName()).processEvent(request, response, input);
  }


  public ResourceOutput serveResource(HttpServletRequest request,
      HttpServletResponse response, ResourceInput input)
      throws PortletContainerException {
    return findPluginByPAPPName(input.getInternalWindowID().
        getPortletApplicationName()).serveResource(request, response, input);
  }

  public RenderOutput render(HttpServletRequest request,
      HttpServletResponse response, RenderInput input)
      throws PortletContainerException {
    return findPluginByPAPPName(input.getInternalWindowID().getPortletApplicationName()).render(request, response, input);
  }

  // sessions replication
  public void sendAttrs(HttpServletRequest request, HttpServletResponse response,
      Map attrs, String portletApplicationName) throws PortletContainerException {
    findPluginByPAPPName(portletApplicationName).sendAttrs(request,response,attrs,portletApplicationName);
  }
  
  public boolean isEventPayloadTypeMatches(String portletAppName, QName eventName, Object payload) {
    return findPluginByPAPPName(portletAppName).isEventPayloadTypeMatches(portletAppName, eventName, payload);
  }

}
