/*
 * Copyright 2001-2007 The eXo platform SAS  All rights reserved.
 * Please look at license.txt in info directory for more license detail. 
 *  
 * Created on 6 janv. 2004
 */
package org.exoplatform.services.portletcontainer.plugins.pc;

import java.util.List;
import java.util.Locale;
import java.util.ResourceBundle;
import java.util.Map;

import javax.xml.namespace.QName;

import org.exoplatform.container.ExoContainer;
import org.exoplatform.services.portletcontainer.pci.PortletData;
import org.exoplatform.services.portletcontainer.pci.model.*;
import org.exoplatform.services.portletcontainer.plugins.pc.portletAPIImp.bundle.ResourceBundleManager;


/**
 * @author Mestrallet Benjamin
 *         benjmestrallet@users.sourceforge.net
 */
public class PortletDataImp implements PortletData {

  protected List userAttributes_;
  protected Portlet portlet_;
  protected UserDataConstraint userDataConstraintType_;
  protected ExoContainer cont;

  
  public PortletDataImp(ExoContainer cont, 
                        Portlet portlet,
                        UserDataConstraint userDataConstraintType,
                        List userAttributes) {
    this.cont = cont;
    this.portlet_ = portlet;
    this.userDataConstraintType_ = userDataConstraintType;
    this.userAttributes_ = userAttributes;
  }

  public Portlet getWrappedPortletTyped() { return portlet_; }

  public List getDisplayName() { return portlet_.getDisplayName(); }

  public List getSecurityRoleRef() { return portlet_.getSecurityRoleRef(); }

  public List getInitParam() {  return portlet_.getInitParam(); }

  public ResourceBundle getPortletInfo(Locale locale) {
    ResourceBundleManager manager = 
      (ResourceBundleManager)cont.getComponentInstanceOfType(ResourceBundleManager.class);
    try {
      return manager.lookupBundle(portlet_, locale);
    } catch (Exception e) {
      return null;
    }
  }

  public List getSupports() { return portlet_.getSupports(); }

  public List getDescription() { return portlet_.getDescription(); }
  
  public String getDescription(String lang) { return portlet_.getDescription(lang) ; }

  public boolean isCacheGlobal() {
    if ("true".equalsIgnoreCase(portlet_.getGlobalCache())) {
      return true;
    }     
    return false;
  }

  public String getExpirationCache() {
    Integer s = portlet_.getCaching();
    if (s == null) return "0";
    return s.toString();
  }

  public String getPortletName() {  return portlet_.getPortletName(); }

  public List getSupportedLocale() { return portlet_.getSupportedLocale(); }

  public ExoPortletPreferences getPortletPreferences() {
    return portlet_.getPortletPreferences();
  }

  public boolean isSecure() {
    if (userDataConstraintType_ != null)  return true;
    return false;
  }

  public List getUserAttributes() { return userAttributes_; }

  public List<QName> getSupportedProcessingEvent() { 
    if (portlet_ == null) return null; // DONE for avoid exc for WSRPAdminPortlet
    return portlet_.getSupportedProcessingEvent(); }

  public List<QName> getSupportedPublishingEvent() { return portlet_.getSupportedPublishingEvent(); }

  public List getSupportedPublicRenderParameter() {
    if (portlet_ == null) return null; // DONE for avoid exc for WSRPAdminPortlet
    return portlet_.getSupportedPublicRenderParameter(); }
  
  public Map getContainerRuntimeOption() { return portlet_.getContainerRuntimeOption(); }

  public boolean getEscapeXml() { 
    if (portlet_ == null) return true; // DONE for avoid exc for WSRPAdminPortlet
    return portlet_.getEscapeXml(); }

}