/**
 * Copyright 2001-2007 The eXo Platform SAS          All rights reserved.
 * Please look at license.txt in info directory for more license detail.
 **/
package org.exoplatform.services.portletcontainer.plugins.pc.portletAPIImp;


import java.util.Iterator;
import java.util.List;
import java.util.Set;

import javax.portlet.PortletURLGenerationListener;
import javax.portlet.ResourceURL;

import org.exoplatform.Constants;
import org.exoplatform.services.portletcontainer.pci.WindowID;
import org.exoplatform.services.portletcontainer.pci.model.Portlet;

/**
 * Created by The eXo Platform SAS 
 * Author : Mestrallet Benjamin
 *          benjmestrallet@users.sourceforge.net
 * Date: Jul 29, 2003
 * Time: 11:13:44 PM
 */
public class ResourceURLImp extends BaseURLImp implements ResourceURL {

  private WindowID windowID;
  private String baseURL;
  
  protected String resourceID;
  protected String originalCacheLevel;
  protected String cacheLevel;
  private final Portlet portletDatas_;
  
  public ResourceURLImp(String type, 
      String markup, 
      boolean isCurrentlySecured, 
      String baseURL, 
      WindowID windowID,
      boolean defaultEscapeXml,
      String cacheLevel){
    this(type, markup, isCurrentlySecured, baseURL, windowID, defaultEscapeXml, cacheLevel, null);
  }
  
  public ResourceURLImp(String type, 
                        String markup, 
                        boolean isCurrentlySecured, 
                        String baseURL, 
                        WindowID windowID,
                        boolean defaultEscapeXml,
                        String cacheLevel,
                        Portlet portletDatas_){
    super(type, markup, isCurrentlySecured, defaultEscapeXml);
    this.baseURL = baseURL;
    this.windowID = windowID;      
    this.cacheLevel = cacheLevel;
    this.originalCacheLevel = cacheLevel;
    this.portletDatas_ = portletDatas_;
  }
  
  protected void invokeFilterResourceURL() {
    if (portletDatas_ == null)
      return;
    List<PortletURLGenerationListener> list = portletDatas_.getUrlListeners();
    if (list == null)
      return;
    for (Iterator<PortletURLGenerationListener> i = list.iterator(); i.hasNext(); ) {
      PortletURLGenerationListener listener = i.next();
      try {
        listener.filterResourceURL(this);
      } catch (Exception e) {}
    }
  }
  
  public String toString(boolean escapeXML) {
    invokeFilterResourceURL();
    
    if(!setSecureCalled && isCurrentlySecured)
      isSecure = true;

    StringBuffer sB = new StringBuffer();
    sB.append(baseURL);
    //sB.append(Constants.AMPERSAND);
    //sB.append(Constants.COMPONENT_PARAMETER);
    //sB.append("=");
    //sB.append(windowID.generateKey());
    sB.append(Constants.AMPERSAND);
    sB.append(Constants.TYPE_PARAMETER);
    sB.append("=");
    sB.append(type);
    sB.append(Constants.AMPERSAND);
    sB.append(Constants.SECURE_PARAMETER);
    sB.append("=");
    sB.append(isSecure);
    
    // The PORTLET_MODE_PARAMETER and WINDOW_STATE_PARAMETER here is not probably necessary.
    
    if (resourceID != null) {
      sB.append(Constants.AMPERSAND);
      sB.append(Constants.RESOURCE_ID_PARAMETER);
      sB.append("=");
      sB.append(resourceID);
    }

    if (cacheLevel != null) {
      sB.append(Constants.AMPERSAND);
      sB.append(Constants.CACHELEVEL_PARAMETER);
      sB.append("=");
      sB.append(cacheLevel);
    }
    
    try {
      Set names = parameters.keySet();
      for (Iterator iterator = names.iterator(); iterator.hasNext();) {
        String name = (String) iterator.next() ;
        Object obj =  parameters.get(name) ;
        if (obj instanceof String) {
          String value = (String) obj ;
          sB.append(Constants.AMPERSAND);
          sB.append(_encode(name, escapeXML));
          sB.append("=");
          sB.append(_encode(value, escapeXML));
        } else {
          String[] values = (String[]) obj ;
          for (int i=0; i < values.length ; i++) {
            sB.append(Constants.AMPERSAND);
            sB.append(_encode(name, escapeXML));
            sB.append("=");
            sB.append(_encode(values[i], escapeXML));
          }
        }
      }
    } catch (Exception e) {
      e.printStackTrace();
    }    
    String propertyString = getPropertyString();
    if (propertyString != "" && propertyString != null)
      {
      //sB.append(Constants.AMPERSAND);
      sB.append(propertyString);
      }
    return sB.toString();
  }
  
  public void setResourceID(String resourceID) {
    this.resourceID = resourceID;
  }

  public String getCacheability() {
    return this.cacheLevel;
  }
  
  public void setCacheability(String cacheLevel_)  {
    if (this.originalCacheLevel.equals(FULL) && (!cacheLevel_.equals(FULL)))
    {
      throw new IllegalStateException("Cacheability level error");
      
    } else if (this.originalCacheLevel.equals(PORTLET) && cacheLevel_.equals(PAGE)) {
        throw new IllegalStateException("Cacheability level error");
    }
    else
    {
      this.cacheLevel = cacheLevel_;
    }    
  }
  
  
}
