/***************************************************************************
 * Copyright 2001-2007 The eXo Platform SAS          All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.services.portletcontainer.plugins.pc.portletAPIImp;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.portlet.PortletMode;
import javax.portlet.PortletModeException;
import javax.portlet.StateAwareResponse;
import javax.portlet.WindowState;
import javax.portlet.WindowStateException;
import javax.xml.bind.JAXBContext;
import javax.xml.namespace.QName;

import org.exoplatform.container.ExoContainer;
import org.exoplatform.services.portletcontainer.PortletContainerConf;
import org.exoplatform.services.portletcontainer.pci.EventOutput;
import org.exoplatform.services.portletcontainer.pci.Input;
import org.exoplatform.services.portletcontainer.pci.model.Portlet;
import org.exoplatform.services.portletcontainer.pci.model.Supports;

/**
 * Created by The eXo Platform SAS
 * Author : Alexey Zavizionov
 *          alexey.zavizionov@exoplatform.com.ua
 * 25.05.2007  
 */
public class StateAwareResponseImp extends PortletResponseImp implements StateAwareResponse {

//  protected List customWindowStates_;//TODO
  private Input input_;
  private Portlet portletDatas_;
  
  public StateAwareResponseImp(ResponseContext resCtx) {
    super(resCtx);
    this.cont_ = resCtx.getCont();
    this.input_ = resCtx.getInput();
    this.customWindowStates_ = resCtx.getCustomWindowState();
    this.redirectionPossible_ = true;
    this.sendRedirectAlreadyOccured_ = false;
    this.portletDatas_ = resCtx.getPortletDatas();
  }

  public void setWindowState(WindowState windowState) throws WindowStateException {  
    if (sendRedirectAlreadyOccured_)
      throw new IllegalStateException("sendRedirect was already called");                         
    if (windowState == null) {
      throw new WindowStateException("The portlet mode is null", windowState); 
    }               
    if (windowState == WindowState.NORMAL || windowState == WindowState.MINIMIZED ||
        windowState == WindowState.MAXIMIZED) {      
      ((EventOutput) this.getOutput()).setNextState(windowState);
      redirectionPossible_ = false;
      return;
    }
    ExoContainer manager = cont_;
    Enumeration e = 
      ((PortletContainerConf)manager.getComponentInstanceOfType(PortletContainerConf.class)).
                                     getSupportedWindowStates();
    while (e.hasMoreElements()) {
      WindowState state = (WindowState) e.nextElement();
      if (state.toString().equals(windowState.toString())) {
        ((EventOutput) this.getOutput()).setNextState(windowState);
        redirectionPossible_ = false;
        return;
      }
    }
    throw new WindowStateException("The window state " + windowState.toString() + " is not supported by the portlet container", windowState);           
  }

  public void setPortletMode(PortletMode portletMode) throws PortletModeException {        
    if (sendRedirectAlreadyOccured_)
      throw new IllegalStateException("sendRedirect was already called");                
    if (portletMode == null)
      throw new PortletModeException("The portlet mode is null", portletMode);
    if (portletMode == PortletMode.VIEW) {
      ((EventOutput) this.getOutput()).setNextMode(portletMode);
      redirectionPossible_ = false;
      return;
    }
    List l = portletDatas_.getSupports();
    for (Iterator iterator = l.iterator(); iterator.hasNext();) {
      Supports supports = (Supports) iterator.next();      
      if (input_.getMarkup().equals(supports.getMimeType())) {
        List modeList = supports.getPortletMode();
        for (Iterator iterator1 = modeList.iterator(); iterator1.hasNext();) {
          String modeString = (String) iterator1.next() ;
          modeString = modeString.toLowerCase() ;
          if (modeString != null && modeString.equals(portletMode.toString())) {
            ((EventOutput) this.getOutput()).setNextMode(portletMode);
            redirectionPossible_ = false;
            return;
          }
        }        
      }
    }    
    throw new PortletModeException("The mode " + portletMode.toString() + " is not supported by that portlet",
                                   portletMode);                            
  }

  public void setRenderParameters(Map<String, String[]> map) {
    if (map == null)
      throw new IllegalArgumentException("the map given is null");
    if (map.containsKey(null))
      throw new IllegalArgumentException("the map given contains a null key");
    Set keys = map.keySet();
    for (Iterator iter = keys.iterator(); iter.hasNext();) {
      if (! (iter.next() instanceof String ))
        throw new IllegalArgumentException("the map contains a non String key");
    }
    Collection values = map.values();
    for (Iterator iter = values.iterator(); iter.hasNext();) {
      if (! (iter.next() instanceof String[] ))
        throw new IllegalArgumentException("the map contains a non String[] value");
    }        
    if (sendRedirectAlreadyOccured_)
      throw new IllegalStateException("sendRedirect was already called");    
    redirectionPossible_ = false;
    ((EventOutput) this.getOutput()).setRenderParameters(map);
  }

  public void setRenderParameter(String s, String s1) {
    if (s == null)
      throw new IllegalArgumentException("the key given is null");
    if (s1 == null)
      throw new IllegalArgumentException("the value given is null");    
    if (sendRedirectAlreadyOccured_)
      throw new IllegalStateException("sendRedirect was already called");    
    redirectionPossible_ = false;
    ((EventOutput) this.getOutput()).setRenderParameter(s, s1);
  }

  public void setRenderParameter(String s, String[] strings) {
    if (s == null)
      throw new IllegalArgumentException("the key given is null");
    if (strings == null)
      throw new IllegalArgumentException("the value given is null");    
    if (sendRedirectAlreadyOccured_)
      throw new IllegalStateException("sendRedirect was already called");    
    redirectionPossible_ = false;
    ((EventOutput) this.getOutput()).setRenderParameters(s, strings);
  }
  
  public boolean isSendRedirectAlreadyOccured() {
    return sendRedirectAlreadyOccured_;
  }
  
  public PortletMode getPortletMode() {
    return input_.getPortletMode();
  }
  
  public Map getRenderParameterMap() {
    //TODO
    return null;
  }
  
  public WindowState getWindowState() {
    return input_.getWindowState();
  }

  private String[] jaxb8_5_2List = new String[] {
      "java.lang.String",
      "java.math.BigInteger",
      "java.math.BigDecimal",
      "java.util.Calendar",
      "java.util.Date",
      "javax.xml.namespace.QName",
      "java.net.URI",
      "javax.xml.datatype.XMLGregorianCalendar",
      "javax.xml.datatype.Duration",
      "java.awt.Image",
      "javax.activation.DataHandler",
      "javax.xml.transform.Source",
      "java.util.UUID"
    };
  
  protected boolean validateByJAXB8_5_2List(Object o) {
    return java.util.Arrays.asList(jaxb8_5_2List).contains(o.getClass().getName());
  }

  protected boolean validateWithJAXB(Object o) {
    if (o == null)
      return true;
    if (validateByJAXB8_5_2List(o))
      return true;
    try {
      JAXBContext jaxbContext = JAXBContext.newInstance(o.getClass());
      jaxbContext.createMarshaller().marshal(o, new NullOutputStream());
    } catch (Exception e) {
      return false;
    }
    return true;
  }

  public void setEvent(QName name, java.io.Serializable event) {
    if (!validateWithJAXB(event))
      throw new java.lang.IllegalArgumentException("setEvent(): can't get binding of " + event.getClass().getName());
    ((EventOutput) getOutput()).setEvent(name, event);
  }
  
  public void setEvent(String name, java.io.Serializable event) {
    ((EventOutput) getOutput()).setEvent(new QName(portletDatas_.getApplication().getDefaultNamespace(), name), event);
  }
  
  private class NullOutputStream extends java.io.OutputStream {
    public void write(int b) {}
  }

  public void removePublicRenderParameter(String param) {
    ((EventOutput) this.getOutput()).getRenderParameters().remove(param);
  }

}
