/**
 * Copyright 2001-2007 The eXo Platform SAS         All rights reserved. 
 * Please look at license.txt in info directory for more license detail.  
 */

package org.exoplatform.services.wsrp2.utils;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.namespace.QName;
import javax.xml.parsers.DocumentBuilderFactory;

import org.apache.axis.message.MessageElement;
import org.exoplatform.services.portletcontainer.pci.EventImpl;
import org.exoplatform.services.wsrp2.type.Event;
import org.exoplatform.services.wsrp2.type.EventPayload;

/**
 * Created by The eXo Platform SAS
 * Author : Alexey Zavizionov
 *          alexey.zavizionov@exoplatform.com.ua
 * Oct 15, 2007  
 */
public class JAXBEventTransformer {

  // convert from "javax.portlet.Event" to "org.exoplatform.services.wsrp2.type.Event[1]"
  public static Event[] getEventsToWSRP(javax.portlet.Event event) {
    return new Event[]{getEventToWSRP(event)};
  }
  
  // convert from "javax.portlet.Event" to "org.exoplatform.services.wsrp2.type.Event"
  private static Event getEventToWSRP(javax.portlet.Event event) {
    Event newEvent = new Event();
    newEvent.setName(event.getQName());
    if (event.getValue() != null){
      String eventType = event.getValue().getClass().toString().substring("class ".length());
      newEvent.setType(new QName(eventType));
      EventPayload eventPayload = null;
      try {
        eventPayload = new EventPayload();
        org.w3c.dom.Document doc = DocumentBuilderFactory.newInstance().newDocumentBuilder().newDocument();
        Serializable o = event.getValue();
        JAXBContext jaxbContext = JAXBContext.newInstance(o.getClass());
        jaxbContext.createMarshaller().marshal(o, doc);
        org.apache.axis.message.MessageElement messageElement = new MessageElement(doc.getDocumentElement());
        org.apache.axis.message.MessageElement[] anyArray = new MessageElement[]{messageElement};
        eventPayload.set_any(anyArray);
      } catch (JAXBException e) {
        e.printStackTrace();
      } catch (Exception e) {
        e.printStackTrace();
      }        
      newEvent.setPayload(eventPayload);      
    } else {
      
    }
    return newEvent;
  }
  
  // convert from "List<javax.portlet.Event>" to "org.exoplatform.services.wsrp2.type.Event[]"
  public static Event[] getEventsToWSRP(List<javax.portlet.Event> eventsList) {
    Event[] events = new Event[eventsList.size()];
    int i=0;
    for (javax.portlet.Event event : eventsList) {
      events[i++] = getEventToWSRP(event);
    }
    return events;
  }
  
  // convert from "org.exoplatform.services.wsrp2.type.Event[]" to "List<javax.portlet.Event>"
  public static List<javax.portlet.Event> getEventsFromWSRP(Event[] eventArray) {
    if (eventArray == null)
      return null;
    List<javax.portlet.Event> eventsList = new ArrayList<javax.portlet.Event>();
    int i=0;
    for (Event event : eventArray) {
      Object o = new Object();
      try {
        String clazz = event.getType().getLocalPart();
        String pkg = clazz.substring(0, clazz.lastIndexOf(".")); 
        ClassLoader cle = Thread.currentThread().getContextClassLoader();//was: this.getClass().getClassLoader();

        JAXBContext jaxbContext = JAXBContext.newInstance(
            pkg,
            cle);
        org.w3c.dom.Element messageElement = event.getPayload().get_any()[0];
        o = jaxbContext.createUnmarshaller().unmarshal(messageElement);
      } catch (JAXBException e) {
        e.printStackTrace();
      } catch (Exception e) {
        e.printStackTrace();
      }
      javax.portlet.Event ev = new EventImpl(event.getName(), o!=null?(Serializable)o:null);
      eventsList.add(ev);
      i++;
    }
    return eventsList;
  }
  
}
